using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class ForkingPath : MonoBehaviour {
	
	public int width;
	public int depth;
	public int companionX;
	public int companionY;
	public Transform [] prefabs;
	public Transform companionPrefab;
	public AudioClip shatterSound;
	
	private Transform _player;
	private PathSegment _currentPath;
	private PathSegment _leftPath;
	private PathSegment _rightPath;
	private PathSegment _fallingPath;
	private DistanceVolumeController _companionVolumeController;
	private Bridge _bridge;
	
	private DayNight _dayNight;
	private bool _timeChosen;
	private Companion _collectedCompanion;
	private bool _homeStretch;
	
	private WordBalloons _wordBalloons;
	
	// Use this for initialization
	void Start () {
		
		_wordBalloons = GetComponent <WordBalloons> ();
		
		_player = GameObject.FindGameObjectWithTag ( "Player" ).transform;
		
		_currentPath = new PathSegment ( null, prefabs, Vector3.zero, width, depth );
		_currentPath.definition = new PathDefinition ( companionX, companionY, width, depth );
		
		for ( int i = 0; i < 3; i ++ )
		{
			Gene gene = new Gene ( _currentPath.definition );
			gene.x = UnityEngine.Random.Range ( -1, 2 ) + ( width - 1 ) / 2;
			gene.y = UnityEngine.Random.Range ( -1, 2 ) + i * 3;
			gene.width = 1;
			gene.height = 1;
			gene.type = 1;
			
			_currentPath.definition.addGene ( gene );
		}
		
		_currentPath.generate ();
		
		_companionVolumeController = GetComponent <DistanceVolumeController> ();

		fork ( true );
		
		_timeChosen = false;
		_dayNight = GetComponent <DayNight> ();
		
		_homeStretch = false;
	}
	
	private void fork ( bool argZigzag )
	{
		int offset = (int) ( ( width + 1f ) / 2f );
		int middle = ( width - 1 ) / 2;
		
		// left
		_leftPath = new PathSegment ( _collectedCompanion == null ? companionPrefab : null, prefabs, _currentPath.localToWorld ( -offset + middle, depth + offset ), width, depth );
		_leftPath.definition = _currentPath.definition.mutate ();
		_leftPath.generate ();
		
		// right
		_rightPath = new PathSegment ( _collectedCompanion == null ? companionPrefab : null, prefabs, _currentPath.localToWorld ( offset + middle, depth + offset ), width, depth );
		_rightPath.definition = _currentPath.definition.mutate ();
		_rightPath.generate ();
		
		if ( argZigzag )
		{
			Zigzag zigzag = new Zigzag ( prefabs [ 0 ], _currentPath.localToWorld ( middle, depth ), width, offset );
			zigzag.generate ();
		}
		else
		{
			// bridge
		
			_bridge = new Bridge ( prefabs, _currentPath.localToWorld ( middle, depth ), width * 2 + 1, offset );
			_bridge.generate ();
		}
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		if ( _player.position.y < -20 )
		{
			Application.LoadLevel ( "Lose" );
		}
		else if ( _player.position.z > _leftPath.origin.z )
		{
			advance ();
		}
		
		if ( _fallingPath != null )
		{
			_fallingPath.FixedUpdate ();
		}
		
		_currentPath.FixedUpdate ();
		_leftPath.FixedUpdate ();
		_rightPath.FixedUpdate ();
		
		if ( _collectedCompanion == null )
		{	
			if ( _currentPath.companion != null && _currentPath.companion.GetComponent <Companion> ().collected )
			{
				_collectedCompanion = _currentPath.companion.GetComponent <Companion> ();
				
				if ( _collectedCompanion != null )
				{
					setCompanionVolumeController ();
					if ( _leftPath.companion != null )
					{
						GameObject.Destroy ( _leftPath.companion );
						_leftPath.companion = null;
					}
					if ( _rightPath.companion != null )
					{
						GameObject.Destroy ( _rightPath.companion );
						_rightPath.companion = null;
					}
				}
			}
		}
	}
	
	public void onCompanionLost ()
	{
		_collectedCompanion = null;
		setCompanionVolumeController ();
		_homeStretch = false;
	}
	
	public void setCompanionVolumeController ()
	{
		if ( _collectedCompanion != null )
		{
			_companionVolumeController.target = _collectedCompanion.transform;
		}
		else if ( _currentPath.companion == null )
		{
			_companionVolumeController.target = null;
		}
		else
		{
			_companionVolumeController.target = _currentPath.companion.transform;
		}
	}
	
	private void advance ()
	{
		if ( _homeStretch )
		{
			Application.LoadLevel ( "Win" );
		}
		else
		{
		
			// left or right?
				
			if ( _player.position.x < _currentPath.origin.x )
			{
				_currentPath = _leftPath;
				_rightPath.Destroy ();
				_fallingPath = _rightPath;
				AudioSource.PlayClipAtPoint ( shatterSound, _rightPath.origin );
			} else {
				_currentPath = _rightPath;
				_leftPath.Destroy ();
				_fallingPath = _leftPath;
				AudioSource.PlayClipAtPoint ( shatterSound, _leftPath.origin );
			}
			
			fork ( _collectedCompanion != null );
			
			setCompanionVolumeController ();
			
			if ( _collectedCompanion != null )
			{
				_dayNight.origin = _currentPath.origin;
				_dayNight.locked = false;
				_homeStretch = true;
			}
			
			if ( ! _timeChosen )
			{
				_timeChosen = true;
				_dayNight.locked = true;
			}
		}
	}
}
