using UnityEngine;
using System.Collections;

public class MusicManager : MonoBehaviour {
	
	public MusicSection [] sections;
	public float bpm;
	public float signature;
	public float sectionMeasures;
	public float minWaitTime;
	public float maxWaitTime;
	
	private int _sectionIndex;
	
	private float _sectionTime = 100;
	
	private bool _waiting;
	
	private float _waitTimeLeft;
	
	private float [] _volumes;
	
	private AudioSource [,] _audioSources;
	
	// Use this for initialization
	void Start () {
		_waiting = true;
		_waitTimeLeft = 0;
		
		float channelCount = 0;
		
		foreach ( MusicSection section in sections )
		{
			channelCount = Mathf.Max ( section.clips.Length );
		}
		
		_volumes = new float [ (int) channelCount ];
		_audioSources = new AudioSource [ sections.Length, (int) channelCount ];
		
		for ( int i = 0; i < channelCount; i ++ )
		{
			_volumes [ i ] = 1;
			
			for ( int s = 0; s < sections.Length; s ++ )
			{
				_audioSources [ s, i ] = gameObject.AddComponent <AudioSource> ();
			}
		}
	}
	
	public void setVolume ( int argChannel, float argVolume )
	{
		if ( argChannel >= 0 && argChannel < _volumes.Length )
		{
			_volumes [ argChannel ] = argVolume;
			
			for ( int s = 0; s < sections.Length; s ++ )
			{
				_audioSources [ s, argChannel ].volume = argVolume;
			}
		}
	}
	
	private void playSection ( int argSectionIndex )
	{
		_sectionIndex = argSectionIndex;
		_sectionTime = 0;
		for ( int i = 0; i < _volumes.Length; i ++ )
		{
			AudioClip clip = sections [ argSectionIndex ].clips [ i ];
			if ( clip != null )
			{
				AudioSource source = _audioSources [ argSectionIndex, i ];
				source.clip = clip;
				source.volume = _volumes [ i ];
				source.Play ();
			}
		}
	}
	
	// Update is called once per frame
	void Update () {
		if ( _waiting )
		{
			_waitTimeLeft -= Time.deltaTime;
			
			if ( _waitTimeLeft <= 0 )
			{
				_waiting = false;
				playSection ( 0 );
			}
		} else {
			_sectionTime += Time.deltaTime * bpm / 60 / signature / sectionMeasures;

			if ( _sectionTime >= 1 )
			{
				var roll = UnityEngine.Random.value;
				
				if ( roll < sections [ _sectionIndex ].backtrackProbability )
				{
					_sectionIndex = Mathf.Max ( _sectionIndex - 1, 0 );
				} 
				else if ( 1 - roll < sections [ _sectionIndex ].advanceProbability ) 
				{
					_sectionIndex ++;
				}
				
				if ( _sectionIndex < sections.Length )
				{
					playSection ( _sectionIndex );
				} 
				else 
				{
					_waiting = true;
					_waitTimeLeft = UnityEngine.Random.value * ( maxWaitTime - minWaitTime ) + minWaitTime;
				}
			}
		}
	}
}
