using System;
using System.Collections.Generic;
using UnityEngine;
	
public class PathDefinition
{
	private const float REMOVE_PROBABILITY = 0.1f;
	private const float SPAWN_PROBABILITY = 0.5f;
	private const float ADD_PROBABILITY = 0.9f;
	private const float SHUFFLE_PROBABILITY = 0.2f;
	
	private const int STARTING_WIDTH = 5;
	//private const int STARTING_WIDTH = 13;
	
	public const int TILE_TYPES = 3;
	private const int TILE_EMPTY = 0;
	private const int TILE_BLANK = 1;
	private const int TILE_TREE = 2;
	
	public int companionX;
	public int companionY;
	
	private float _width;
	private float _height;
	
	private List <Gene> _genes;
	
	public PathDefinition ( int argCompanionX, int argCompanionY, float argWidth, float argHeight )
	{
		_width = argWidth;
		_height = argHeight;
		companionX = argCompanionX;
		companionY = argCompanionY;
		
		_genes = new List <Gene> ();
	}
	
	public float width
	{
		get
		{
			return _width;
		}
	}
	
	public float height
	{
		get
		{
			return _height;
		}
	}
	
	public TileDefinition getTileAt ( int argX, int argY )
	{
		if ( argX == companionX && argY == companionY )
		{
			return new TileDefinition ( 1, Color.gray );
		}
		
		foreach ( Gene gene in _genes )
		{
			int dx = Mathf.Abs ( gene.x - argX );
			int dy = Mathf.Abs ( gene.y - argY );
			
			if ( dx <= gene.width && dy <= gene.height )
			{
				return new TileDefinition ( gene.type, gene.color );
			}
		}
		
		if ( Mathf.Abs ( argX - ( ( _width - 1 ) / 2 ) ) < ( ( STARTING_WIDTH + 1 ) / 2 ) )
		{
			return new TileDefinition ( 1, Color.gray );
		}
		else
		{
			return new TileDefinition ( 0, Color.gray );
		}
	}
	
	public PathDefinition mutate ()
	{
		PathDefinition result = new PathDefinition ( companionX, companionY, _width, _height );
		
		foreach ( Gene gene in _genes )
		{
			result._genes.Add ( gene.mutate () );
		}
		
		if ( result._genes.Count == 0 )
		{
			result._genes.Add ( new Gene ( this ) );
		}
		else
		{
			// remove random genes
			int i = 0;
			while ( i < result._genes.Count )
			{
				if ( UnityEngine.Random.value < REMOVE_PROBABILITY )
				{
					result._genes.RemoveAt ( i );
				}
				else
				{
					i ++;
				}
			}
			
			// spawn new genes
			i = 0;
			while ( i < result._genes.Count )
			{
				if ( UnityEngine.Random.value < SPAWN_PROBABILITY )
				{
					result._genes.Add ( result._genes [ i ].mutate () );
				}
				
				i ++;
			}
			
			// add whole new gene
			
			if ( UnityEngine.Random.value < ADD_PROBABILITY )
			{
				result._genes.Add ( new Gene ( this ) );
			}
			
			// shuffle
			
			for ( i = 0; i < result._genes.Count - 1; i ++ )
			{
				if ( UnityEngine.Random.value < SHUFFLE_PROBABILITY )
				{
					Gene temp = result._genes [ i ];
					result._genes [ i ] = result._genes [ i + 1 ];
					result._genes [ i + 1 ] = temp;
				}
			}
		}
		
		return result;
	}
	
	public void addGene ( Gene argGene )
	{
		_genes.Add ( argGene );
	}
}

