using System;
using UnityEngine;

public class PathSegment : TileBlock
{
	public PathDefinition definition;
	public Transform [] prefabs;
	public Transform companionPrefab;
	public GameObject companion;
	
	private bool _destroyed;
	private bool _creating;
	
	private int _creatingZ;
	
	public PathSegment ( Transform argCompanionPrefab, Transform [] argPrefabs, Vector3 argOrigin, int argWidth, int argHeight ) : base ( argOrigin, argWidth, argHeight )
	{
		prefabs = argPrefabs;
		companionPrefab = argCompanionPrefab;
		_destroyed = false;
		_creating = false;
	}
	
	public void generate ()
	{
		_creating = true;
		_creatingZ = 0;
		
		if ( companionPrefab != null )
		{
			companion = ( (Transform ) GameObject.Instantiate ( 
				companionPrefab,
				localToWorld ( definition.companionX, definition.companionY ) + new Vector3 ( 0f, 10f, 0f ),
				Quaternion.identity
			) ).gameObject;
		}
	}
	
	public void Destroy ()
	{
		_destroyed = true;
	}
	
	public void FixedUpdate ()
	{
		if ( _creating )
		{
			for ( int x = 0; x < width; x ++ )
			{
				
				TileDefinition tile = definition.getTileAt ( x, _creatingZ );
				
				if ( tile.type > 0 )
				{
					placeTile ( prefabs [ tile.type - 1 ], x, _creatingZ, tile.color );
				}
			}
			
			_creatingZ ++;
			
			if ( _creatingZ == depth )
			{
				_creating = false;
			}
		}
		else if ( _destroyed )
		{
			for ( int i = 0; i < 5; i ++ )
			{
				if ( tiles.Count > 0 )
				{
				
					GameObject tile = tiles [ 0 ];
					tiles.RemoveAt ( 0 );
					tile.GetComponent <FallingTile> ().fall ();
				}
			}
		}
	}
}

